@@ -4,10 +4,12 @@ use common::{
     develop, errors, get_python_implementation, handle_result, integration, other, test_python_path,
 };
 use indoc::indoc;
+use maturin::pyproject_toml::SdistGenerator;
 use maturin::Target;
 use std::env;
 use std::path::{Path, PathBuf};
 use time::macros::datetime;
+use which::which;
 
 mod common;
 
@@ -25,7 +27,7 @@ fn develop_pyo3_pure() {
 #[ignore]
 fn develop_pyo3_pure_conda() {
     // Only run on GitHub Actions for now
-    if std::env::var("GITHUB_ACTIONS").is_ok() {
+    if env::var("GITHUB_ACTIONS").is_ok() {
         handle_result(develop::test_develop(
             "test-crates/pyo3-pure",
             None,
@@ -107,22 +109,26 @@ fn develop_cffi_mixed() {
 
 #[test]
 fn develop_uniffi_pure() {
-    handle_result(develop::test_develop(
-        "test-crates/uniffi-pure",
-        None,
-        "develop-uniffi-pure",
-        false,
-    ));
+    if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
+        handle_result(develop::test_develop(
+            "test-crates/uniffi-pure",
+            None,
+            "develop-uniffi-pure",
+            false,
+        ));
+    }
 }
 
 #[test]
 fn develop_uniffi_mixed() {
-    handle_result(develop::test_develop(
-        "test-crates/uniffi-mixed",
-        None,
-        "develop-uniffi-mixed",
-        false,
-    ));
+    if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
+        handle_result(develop::test_develop(
+            "test-crates/uniffi-mixed",
+            None,
+            "develop-uniffi-mixed",
+            false,
+        ));
+    }
 }
 
 #[test]
@@ -152,8 +158,8 @@ fn integration_pyo3_bin() {
         target.get_python()
     });
     let python_implementation = get_python_implementation(&python).unwrap();
-    if python_implementation == "pypy" {
-        // PyPy doesn't support the 'auto-initialize' feature of pyo3
+    if python_implementation == "pypy" || python_implementation == "graalpy" {
+        // PyPy & GraalPy do not support the 'auto-initialize' feature of pyo3
         return;
     }
 
@@ -236,7 +242,7 @@ fn integration_pyo3_mixed_src_layout() {
 #[cfg_attr(target_os = "macos", ignore)] // Don't run it on macOS, too slow
 fn integration_pyo3_pure_conda() {
     // Only run on GitHub Actions for now
-    if std::env::var("GITHUB_ACTIONS").is_ok() {
+    if env::var("GITHUB_ACTIONS").is_ok() {
         handle_result(integration::test_integration_conda(
             "test-crates/pyo3-mixed",
             None,
@@ -268,24 +274,28 @@ fn integration_cffi_mixed() {
 
 #[test]
 fn integration_uniffi_pure() {
-    handle_result(integration::test_integration(
-        "test-crates/uniffi-pure",
-        None,
-        "integration-uniffi-pure",
-        false,
-        None,
-    ));
+    if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
+        handle_result(integration::test_integration(
+            "test-crates/uniffi-pure",
+            None,
+            "integration-uniffi-pure",
+            false,
+            None,
+        ));
+    }
 }
 
 #[test]
 fn integration_uniffi_mixed() {
-    handle_result(integration::test_integration(
-        "test-crates/uniffi-mixed",
-        None,
-        "integration-uniffi-mixed",
-        false,
-        None,
-    ));
+    if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
+        handle_result(integration::test_integration(
+            "test-crates/uniffi-mixed",
+            None,
+            "integration-uniffi-mixed",
+            false,
+            None,
+        ));
+    }
 }
 
 #[test]
@@ -322,7 +332,7 @@ fn integration_with_data() {
 }
 
 #[test]
-// Sourced from https://pypi.org/project/wasmtime/5.0.0/#files
+// Sourced from https://pypi.org/project/wasmtime/7.0.0/#files
 // update with wasmtime updates
 #[cfg(any(
     all(target_os = "windows", target_arch = "x86_64"),
@@ -373,7 +383,23 @@ fn abi3_without_version() {
 }
 
 #[test]
-#[cfg_attr(not(all(target_os = "linux", target_env = "gnu")), ignore)]
+// Only run this test on platforms that has manylinux support
+#[cfg_attr(
+    not(all(
+        target_os = "linux",
+        target_env = "gnu",
+        any(
+            target_arch = "i686",
+            target_arch = "x86_64",
+            target_arch = "aarch64",
+            target_arch = "powerpc64",
+            target_arch = "powerpc64le",
+            target_arch = "s390x",
+            target_arch = "armv7"
+        )
+    )),
+    ignore
+)]
 fn pyo3_no_extension_module() {
     let python = test_python_path().map(PathBuf::from).unwrap_or_else(|| {
         let target = Target::from_target_triple(None).unwrap();
@@ -396,6 +422,11 @@ fn invalid_manylinux_does_not_panic() {
     handle_result(errors::invalid_manylinux_does_not_panic())
 }
 
+#[test]
+fn warn_on_missing_python_source() {
+    handle_result(errors::warn_on_missing_python_source())
+}
+
 #[test]
 #[cfg_attr(not(target_os = "linux"), ignore)]
 fn musl() {
@@ -423,7 +454,7 @@ fn workspace_members_non_local_dep_sdist() {
         license = "MIT"
 
         [dependencies]
-        pyo3 = { version = "0.18.0", features = ["abi3-py37", "extension-module", "generate-import-lib"] }
+        pyo3 = { version = "0.19.0", features = ["abi3-py37", "extension-module", "generate-import-lib"] }
 
         [lib]
         name = "pyo3_pure"
@@ -432,6 +463,7 @@ fn workspace_members_non_local_dep_sdist() {
     );
     handle_result(other::test_source_distribution(
         "test-crates/pyo3-pure",
+        SdistGenerator::Cargo,
         vec![
             "pyo3_pure-0.1.0+abc123de/Cargo.lock",
             "pyo3_pure-0.1.0+abc123de/Cargo.toml",
@@ -454,6 +486,7 @@ fn workspace_members_non_local_dep_sdist() {
 fn lib_with_path_dep_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/sdist_with_path_dep",
+        SdistGenerator::Cargo,
         vec![
             "sdist_with_path_dep-0.1.0/local_dependencies/some_path_dep/Cargo.toml",
             "sdist_with_path_dep-0.1.0/local_dependencies/some_path_dep/src/lib.rs",
@@ -470,10 +503,54 @@ fn lib_with_path_dep_sdist() {
     ))
 }
 
+#[test]
+fn lib_with_target_path_dep() {
+    let cargo_toml = indoc!(
+        r#"
+        [package]
+        name = "sdist_with_target_path_dep"
+        version = "0.1.0"
+        authors = ["konstin <konstin@mailbox.org>"]
+        edition = "2021"
+
+        [lib]
+        crate-type = ["cdylib"]
+
+        [dependencies]
+        # Don't use the macros feature, which makes compilation much faster
+        pyo3 = { version = "0.19.0", default-features = false, features = ["extension-module"] }
+
+        [target.'cfg(not(target_endian = "all-over-the-place"))'.dependencies]
+        some_path_dep = { path = "local_dependencies/some_path_dep" }
+        "#
+    );
+    handle_result(other::test_source_distribution(
+        "test-crates/sdist_with_target_path_dep",
+        SdistGenerator::Cargo,
+        vec![
+            "sdist_with_target_path_dep-0.1.0/local_dependencies/some_path_dep/Cargo.toml",
+            "sdist_with_target_path_dep-0.1.0/local_dependencies/some_path_dep/src/lib.rs",
+            "sdist_with_target_path_dep-0.1.0/local_dependencies/transitive_path_dep/Cargo.toml",
+            "sdist_with_target_path_dep-0.1.0/local_dependencies/transitive_path_dep/src/lib.rs",
+            "sdist_with_target_path_dep-0.1.0/Cargo.toml",
+            "sdist_with_target_path_dep-0.1.0/Cargo.lock",
+            "sdist_with_target_path_dep-0.1.0/pyproject.toml",
+            "sdist_with_target_path_dep-0.1.0/src/lib.rs",
+            "sdist_with_target_path_dep-0.1.0/PKG-INFO",
+        ],
+        Some((
+            Path::new("sdist_with_target_path_dep-0.1.0/Cargo.toml"),
+            cargo_toml,
+        )),
+        "sdist-lib-with-target-path-dep",
+    ))
+}
+
 #[test]
 fn pyo3_mixed_src_layout_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/pyo3-mixed-src/rust",
+        SdistGenerator::Cargo,
         vec![
             "pyo3_mixed_src-2.1.3/pyproject.toml",
             "pyo3_mixed_src-2.1.3/src/pyo3_mixed_src/__init__.py",
@@ -494,6 +571,7 @@ fn pyo3_mixed_src_layout_sdist() {
 fn pyo3_mixed_include_exclude_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/pyo3-mixed-include-exclude",
+        SdistGenerator::Cargo,
         vec![
             // "pyo3_mixed_include_exclude-2.1.3/.gitignore", // excluded
             "pyo3_mixed_include_exclude-2.1.3/Cargo.lock",
@@ -516,6 +594,36 @@ fn pyo3_mixed_include_exclude_sdist() {
     ))
 }
 
+#[test]
+fn pyo3_mixed_include_exclude_git_sdist_generator() {
+    if !Path::new(".git").exists() {
+        return;
+    }
+    handle_result(other::test_source_distribution(
+        "test-crates/pyo3-mixed-include-exclude",
+        SdistGenerator::Git,
+        vec![
+            // "pyo3_mixed_include_exclude-2.1.3/.gitignore", // excluded
+            "pyo3_mixed_include_exclude-2.1.3/Cargo.lock",
+            "pyo3_mixed_include_exclude-2.1.3/Cargo.toml",
+            "pyo3_mixed_include_exclude-2.1.3/PKG-INFO",
+            "pyo3_mixed_include_exclude-2.1.3/README.md",
+            "pyo3_mixed_include_exclude-2.1.3/check_installed/check_installed.py",
+            // "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/exclude_this_file, excluded
+            "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/__init__.py",
+            "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/include_this_file", // included
+            "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/python_module/__init__.py",
+            "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/python_module/double.py",
+            "pyo3_mixed_include_exclude-2.1.3/pyproject.toml",
+            "pyo3_mixed_include_exclude-2.1.3/src/lib.rs",
+            // "pyo3_mixed_include_exclude-2.1.3/tests/test_pyo3_mixed_include_exclude.py", excluded
+            "pyo3_mixed_include_exclude-2.1.3/tox.ini",
+        ],
+        None,
+        "sdist-pyo3-mixed-include-exclude-git",
+    ))
+}
+
 #[test]
 fn pyo3_mixed_include_exclude_wheel_files() {
     handle_result(other::check_wheel_files(
@@ -539,6 +647,7 @@ fn pyo3_mixed_include_exclude_wheel_files() {
 fn workspace_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/workspace/py",
+        SdistGenerator::Cargo,
         vec![
             "py-0.1.0/Cargo.lock",
             "py-0.1.0/Cargo.toml",
@@ -555,15 +664,16 @@ fn workspace_sdist() {
 fn workspace_with_path_dep_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/workspace_with_path_dep/python",
+        SdistGenerator::Cargo,
         vec![
             "workspace_with_path_dep-0.1.0/local_dependencies/generic_lib/Cargo.toml",
             "workspace_with_path_dep-0.1.0/local_dependencies/generic_lib/src/lib.rs",
             "workspace_with_path_dep-0.1.0/local_dependencies/transitive_lib/Cargo.toml",
             "workspace_with_path_dep-0.1.0/local_dependencies/transitive_lib/src/lib.rs",
-            "workspace_with_path_dep-0.1.0/Cargo.lock",
-            "workspace_with_path_dep-0.1.0/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/python/Cargo.lock",
+            "workspace_with_path_dep-0.1.0/python/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/python/src/lib.rs",
             "workspace_with_path_dep-0.1.0/pyproject.toml",
-            "workspace_with_path_dep-0.1.0/src/lib.rs",
             "workspace_with_path_dep-0.1.0/PKG-INFO",
         ],
         None,
@@ -571,11 +681,39 @@ fn workspace_with_path_dep_sdist() {
     ))
 }
 
+#[test]
+fn workspace_with_path_dep_git_sdist_generator() {
+    if !Path::new(".git").exists() {
+        return;
+    }
+    handle_result(other::test_source_distribution(
+        "test-crates/workspace_with_path_dep/python",
+        SdistGenerator::Git,
+        vec![
+            "workspace_with_path_dep-0.1.0/Cargo.lock",
+            "workspace_with_path_dep-0.1.0/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/dont_include_in_sdist/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/dont_include_in_sdist/src/main.rs",
+            "workspace_with_path_dep-0.1.0/generic_lib/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/generic_lib/src/lib.rs",
+            "workspace_with_path_dep-0.1.0/pyproject.toml",
+            "workspace_with_path_dep-0.1.0/python/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/python/src/lib.rs",
+            "workspace_with_path_dep-0.1.0/transitive_lib/Cargo.toml",
+            "workspace_with_path_dep-0.1.0/transitive_lib/src/lib.rs",
+            "workspace_with_path_dep-0.1.0/PKG-INFO",
+        ],
+        None,
+        "sdist-workspace-with-path-dep-git",
+    ))
+}
+
 #[rustversion::since(1.64)]
 #[test]
 fn workspace_inheritance_sdist() {
     handle_result(other::test_source_distribution(
         "test-crates/workspace-inheritance/python",
+        SdistGenerator::Cargo,
         vec![
             "workspace_inheritance-0.1.0/local_dependencies/generic_lib/Cargo.toml",
             "workspace_inheritance-0.1.0/local_dependencies/generic_lib/src/lib.rs",
